import os

DB_PATH = os.path.join(os.path.expanduser("~"), ".seasondog", "database")
PLAYER = "mplayer -lavdopts threads=4 -fs {player_args} \"{videofile}\""
SUB_EXTENSIONS = ["srt", "ass", "aqt", "gsub", "sub", "ttxt", "pjs", "psb", "rt", "stl", "ssa", "usf", "idx",]
MOVIE_EXTENSIONS = ["mov", "mpg", "mpeg", "m2v", "mkv", "webm", "ogm", "mp4", "m4v", "mp4v", "vob", "qt", "nuv", "wmv", "asf", "rm", "rmvb", "flc", "avi", "fli", "flv", "gl", "dl", "xcf", "xwd", "yuv", "cgm", "emf", ]
PREFIXES = ["episode", "ep", "e", "x"]
SURROUNDINGS = [" ", "", "_"]

COLORS = {
    "blue": "\033[94m",
    "magenta": "\033[95m",
    "green": "\033[92m",
    "yellow": "\033[93m",
    "grey": "\033[90m",
    "red": "\033[91m",

    "bold": "\033[1m",

    "endc": "\033[0m",
}

COLORS.update({
    "c_title": COLORS["grey"],
    "c_ep_number": COLORS["blue"] + COLORS["bold"],
    "c_path": COLORS["blue"],
    "c_args": COLORS["blue"],
    "c_control": COLORS["grey"],
    "c_success": COLORS["green"],
    "c_error": COLORS["red"],
})

USE_COLORS = True

MATCHER_DEBUG = False

try:
    exec(open(os.path.join(os.path.expanduser("~"), ".sdogrc")).read())
except FileNotFoundError:
    pass

if not USE_COLORS:
    c = {}
    for k, v in COLORS.items():
        c[k] = ""

    COLORS = c



from seasondog.config import MOVIE_EXTENSIONS, SUB_EXTENSIONS
from seasondog import config

import time
import os
import re


def leading_zero(episode, count):
    return ''.join(['0' for i in range(count - len(str(episode)))]) + str(episode)


def re_matcher(pattern):
    return lambda x, n: re.match(pattern, x, re.I)


def simple_matcher(episode, extensions=[], zero=2):
    return re_matcher(r"^.*{}.*({})$".format(leading_zero(episode, zero), "|".join(extensions)))


def prefix_matcher(prefix, episode, extensions=[], zero=2):
    return re_matcher(r"^.*{}{}.*({})$".format(prefix, leading_zero(episode, zero), "|".join(extensions)))


def prefix_suffix_matcher(prefix, suffix, episode, extensions=[], zero=2):
    return re_matcher(r"^.*{}{}{}.*({})$".format(prefix, leading_zero(episode, zero), suffix, "|".join(extensions)))


def nth_matcher(episode):
    return lambda x, n: n == episode


def match(directory, matchers, limit=1):
    if config.MATCHER_DEBUG:
        start_time = time.time()

    for path, dirs, files in os.walk(directory):
        files.sort()

        for matcher in matchers:
            result = []
            i = 0
            for x in files:
                i += 1
                if matcher(x, i):
                    result.append(os.path.join(path, x))

            if config.MATCHER_DEBUG:
                if len(result) != 0:
                    print("Matcher {}, matched {}".format(matcher, result))
                    print("    ", "ok" if len(result) == 1 else "fail: multiple results")

            if result and (limit == -1 or len(result) == limit):
                if config.MATCHER_DEBUG:
                    print("Matched in: {}".format(time.time() - start_time))
                return result

    if config.MATCHER_DEBUG:
        print("Matched in: {}".format(time.time() - start_time))
        print("None matched!")


def match_episode(directory, episode, limit=1):
    matchers = []
    for p in config.PREFIXES:
        for s in config.SURROUNDINGS:
            for s in config.SURROUNDINGS:
                for i in range(1, 3):
                    matchers.append(prefix_suffix_matcher(p+s, s, episode, MOVIE_EXTENSIONS, i))

    for i in range(1, 3):
        matchers.append(simple_matcher(episode, MOVIE_EXTENSIONS, i))
    matchers.append(nth_matcher(episode))

    return match(directory, matchers, limit)


def match_subs(directory, episode, limit=1):
    matchers = []
    for p in config.PREFIXES:
        for s in config.SURROUNDINGS:
            for s in config.SURROUNDINGS:
                for i in range(1, 3):
                    matchers.append(prefix_suffix_matcher(p, s, episode, SUB_EXTENSIONS, i))

    matchers.append(simple_matcher(episode, SUB_EXTENSIONS))
    matchers.append(nth_matcher(episode))

    return match(directory, matchers, limit)


def match_file(directory, episode, limit=1):
    return match(
        directory, [
            simple_matcher(episode),
            nth_matcher(episode),
        ],
        limit)



import os
try:
    import readline
except ImportError:
    pass
from seasondog import config
from seasondog import info
from seasondog import database as database
from seasondog import watcher
from seasondog import runtime as r
from optparse import OptionParser


def init_wizard(runtime, db):
    print(r.format("No record found on {c_path}{}{endc}.", runtime[r.PATH]))
    try:
        data = {database.EPISODE: 0,
                database.PLAYER_ARGS: input("Player args: "),
                }
    except KeyboardInterrupt:
        raise RuntimeError(r.format("{c_error}Cancelled.{endc}"))

    database.set(db, runtime[r.PATH], data)
    return data


def opt_parser():
    parser = OptionParser(epilog="version {}, {}".format(info.VERSION, info.URL))
    parser.set_usage("""{} [action=next,prev,reset,status,set,args]
    n(ext) - watch next episode
    p(rev) - previous
    w(atch) - watch current episode
    set <EPISODE> - set progress
    args <ARGS> - set player args
    r(eset) - reset progress and settings for directory
        -f <PATH> - provide directory PATH instead of current one
    m(igrate) <DESTINATION> - migrate current directory to DESTINATION
        Directory name is preserved, only it's location is changed
        -f <PATH> - provide directory PATH instead of current one
        -p - don't preserve directory name, DESTINATION is full path to directory
    cleanup - remove not-existent directories from database
    s(tatus) - show progress and settings\n""".format(info.NAME))
    parser.add_option("", "--database", help="Provide database location")
    parser.add_option("-a", "--player-args", help="Provide overriding player args")
    parser.add_option("-f", "--from", help="Provide from parameter for migration (instead of using current directory)")
    parser.add_option("-p", "--not-preserve", help="Don't preserve directory name", action="store_true")
    parser.add_option("-v", "--version", help="Show version", action="store_true")
    parser.add_option("-d", "--debug", help="Turn on debug messages", action="store_true")

    return parser


def arg(args, n):
    try:
        return args[n]
    except IndexError:
        raise RuntimeError(r.format("{c_error}Command requires at least {} argument(s)!{endc}", n))


def main():
    (opt, args) = opt_parser().parse_args()
    runtime = r.runtime_struct(os.path.abspath("."), opt)
    db = database.load(runtime[r.DB_PATH])

    if opt.version:
        action = "version"
    elif len(args) > 0:
        action = args[0]
    else:
        action = "next"

    if opt.debug:
        config.MATCHER_DEBUG = True

    try:
        if action == "version":
            print(r.format(
                "seasondog {version}\nCopyright {copy}.\nLicensed under {license}.",
                version=info.VERSION,
                copy=info.COPYRIGHT,
                license=info.LICENSE,
                url=info.URL,))

            return

        if action == "migrate" or action == "m":
            if opt.__dict__["from"]:
                old_path = os.path.abspath(opt.__dict__["from"])
                whatever, dir = os.path.split(old_path)
            else:
                whatever, dir = os.path.split(runtime[r.PATH])
                old_path = os.path.abspath(runtime[r.PATH])

            if opt.not_preserve:
                new_path = os.path.abspath(arg(args, 1))
            else:
                new_path = os.path.abspath(os.path.join(arg(args, 1), dir))

            try:
                input(r.format("New directory for {c_path}{}{endc} is {c_path}{}{endc}. Is this right?\n{c_control}[↵ watch, ^C break]{endc}", old_path, new_path))

                database.migrate(db, old_path, new_path)
            except KeyboardInterrupt:
                pass

            return

        elif action == "cleanup":
            deleted = database.cleanup(db)
            print(r.format("Deleted {c_ep_number}{}{endc} entries.", deleted))

            return

        data = database.get(db, runtime[r.PATH])
        iswatch = True

        if action == "reset" or action == "r":
            path = opt.__dict__["from"] if opt.__dict__["from"] else runtime[r.PATH]
            try:
                database.unset(db, path)
            except KeyError:
                print(r.format("{c_error}There's no record at {c_path}{}{c_error} !{endc}", path))
            database.save(db)

            return

        elif action == "status" or action == "s":
            print(r.format(
                "{blue}{}{endc} v{}, database {} v{} ({grey}{}{endc})\nInternal path: {grey}{}{endc}",
                info.NAME,
                info.VERSION,
                database.NAME,
                database.VERSION,
                db[database.PATH],
                runtime[r.PATH],))

            if data:
                print(r.format("Current episode: {blue}{}{endc}\nPlayer args: {blue}{}{endc}",
                    data[database.EPISODE],
                    data[database.PLAYER_ARGS] or "None",))
            else:
                print(r.format("No data on {c_path}{}{endc}.", runtime[r.PATH]))

            return

        if not data:
            data = init_wizard(runtime, db)
        if action == "next" or action == "n":
            data[database.EPISODE] += 1
        elif action == "prev" or action == "p":
            data[database.EPISODE] -= 1
        elif action == "watch" or action == "w":
            pass
        elif action == "set":
            try:
                data[database.EPISODE] = int(arg(args, 1))
            except ValueError:
                raise RuntimeError(r.format("{c_error}{} is not a number!{endc}", arg(args, 1)))

        elif action == "args":
            data[database.PLAYER_ARGS] = arg(args, 1)
            iswatch = False

        database.set(db, runtime[r.PATH], data)
        database.commit(db)

        if iswatch:
            watcher.watch(runtime, data)
    except RuntimeError as e:
        print(e)
